﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Research.Kinect.Nui;

namespace TestKinect
{
    /// <summary>
    /// Logique d'interaction pour MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // NUI de la Kinect.
        private Runtime kinectRuntime = new Runtime();

        public MainWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Handler d'initialisation de notre fenêtre.
        /// Initialisation de la bibliothèque NUI et préparation de la récupération
        /// du flux vidéo.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // On demande l'accès à la caméra en couleur et à la profondeur.
            kinectRuntime.Initialize(RuntimeOptions.UseColor | RuntimeOptions.UseDepth);

            // On s'abonne à l'événement permettant de récupérer les images.
            kinectRuntime.VideoFrameReady += new EventHandler<ImageFrameReadyEventArgs>(kinectRuntime_VideoFrameReady);

            // On s'abonne à l'événement permettant de récupérer la profondeur.
            kinectRuntime.DepthFrameReady += new EventHandler<ImageFrameReadyEventArgs>(kinectRuntime_DepthFrameReady);

            // On ouvre le flux vidéo :
            // - ImageStreamType.Video indique que nous voulons le flux vidéo;
            // - 2 donne le nombre de buffers que la Runtime peut utiliser;
            // - ImageResolution.Resolution640x480 indique la taille de l'image souhaitée;
            // - ImageType.Color précise que nous la voulons en pixels RGB.
            kinectRuntime.VideoStream.Open(ImageStreamType.Video, 2, ImageResolution.Resolution640x480, ImageType.Color);

            // On ouvre le flux profondeur :
            // - ImageStreamType.Depth indique que nous voulons le flux profondeur;
            // - 2 donne le nombre de buffers que la Runtime peut utiliser;
            // - ImageResolution.Resolution640x480 indique la taille de la pseudo image souhaitée;
            // - ImageType.Depth précise que nous la voulons avec les informations de profondeur.
            kinectRuntime.DepthStream.Open(ImageStreamType.Depth, 2, ImageResolution.Resolution640x480, ImageType.Depth);
        }

        /// <summary>
        /// Handler permettant le traitement des images vidéos fournies par Kinect.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void kinectRuntime_VideoFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            PlanarImage image = e.ImageFrame.Image;
            image1.Source = BitmapSource.Create(image.Width, image.Height, 96, 96, PixelFormats.Bgr32, null, image.Bits, image.Width * image.BytesPerPixel);
        }

        /// <summary>
        /// Handler permettant le traitement des images "profondeurs" fournies par Kinect.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void kinectRuntime_DepthFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            // Création du tableau contenant l'image que nous allons fabriquer.
            // Les dimmensions dépendent de l'"image" reçue.
            byte[] imageAffichage = new byte[e.ImageFrame.Image.Width * e.ImageFrame.Image.Height * 4];

            // Récupérations du détail de l'image reçue.
            byte[] imageRecue = e.ImageFrame.Image.Bits;

            // Traitement des éléments de l'image un par un.
            // iRecu permet le parcours de l'image reçue ou chaque élément occupe 2 octets.
            // iAffiche permet le parcours de l'image que nous afficherons ou chaque élément occupe 4 octets.
            for (int iRecu = 0, iAffiche = 0; (iRecu < imageRecue.Length) && (iAffiche < imageAffichage.Length); iRecu += 2, iAffiche += 4)
            {
                // La vraie distance en mm par rappor à Kinect est sur 12 bits.
                int profondeur = (imageRecue[iRecu + 1] & 0xF) << 8 | imageRecue[iRecu];

                // Pour l'affichage on veut une profondeur sur un octet donc on doit traiter;
                // Ici ce qui est à ProfondeurMin ou moins est blanc,
                // ce qui est à ProfondeurMax ou plus est noir.
                // Ajustez les valeurs en fonction de votre espace.
                const float ProfondeurMin = 800;
                const float ProfondeurMax = 1200;
                const float PlageTraitee = ProfondeurMax - ProfondeurMin;

                byte couleur = (byte)(255 - (255 * Math.Max(Math.Min(profondeur, ProfondeurMax) - ProfondeurMin, 0) / PlageTraitee));
   
                // Pour obtenir des niveaux de gris on met les 3 composantes RGB à la même valeur.
                imageAffichage[iAffiche] = couleur;
                imageAffichage[iAffiche + 1] = couleur;
                imageAffichage[iAffiche + 2] = couleur;
            }
            image2.Source = BitmapSource.Create(e.ImageFrame.Image.Width, e.ImageFrame.Image.Height, 96, 96, PixelFormats.Bgr32, null, imageAffichage, e.ImageFrame.Image.Width * 4);
        }

        /// <summary>
        /// Handler de fin de clôture de notre fenêtre.
        /// Libération des ressources de la bibliothèque NUI.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closed(object sender, EventArgs e)
        {
            // On libère les ressources.
            kinectRuntime.Uninitialize();
        }
    }
}